package grpctool

import (
	"context"
	"log/slog"
	"time"

	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v17/internal/tool/errz"
	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v17/internal/tool/fieldz"
	"google.golang.org/grpc"
)

// PoolSelf is a decorator that uses an in-memory connection to dial self rather than going over network.
type PoolSelf struct {
	log      *slog.Logger
	errRep   errz.ErrReporter
	delegate PoolInterface
	selfURL  string
	conn     selfPoolConn
}

func NewPoolSelf(log *slog.Logger, errRep errz.ErrReporter, delegate PoolInterface, selfURL string, selfConn *grpc.ClientConn) *PoolSelf {
	return &PoolSelf{
		log:      log,
		errRep:   errRep,
		delegate: delegate,
		selfURL:  selfURL,
		conn: selfPoolConn{
			ClientConn: selfConn,
		},
	}
}

func (p *PoolSelf) Dial(ctx context.Context, targetURL string) (PoolConn, error) {
	if targetURL == p.selfURL {
		return &p.conn, nil
	}
	return p.delegate.Dial(ctx, targetURL)
}

func (p *PoolSelf) Shutdown(deadline time.Duration) {
	p.delegate.Shutdown(deadline)
	err := p.conn.Close()
	if err != nil {
		p.errRep.HandleProcessingError(context.Background(), p.log, "Failed to close connection",
			err, fieldz.New("conn_target", p.conn.CanonicalTarget()))
	}
}

type selfPoolConn struct {
	*grpc.ClientConn
}

func (s *selfPoolConn) Done() {
	// noop
}
